@@ -8,7 +8,7 @@ package Tickit::Term;
 use strict;
 use warnings;
 
-our $VERSION = '0.16_001';
+our $VERSION = '0.17_001';
 
 # Load the XS code
 require Tickit;
@@ -68,6 +68,12 @@ Takes the following named arguments at construction time:
 
 =over 8
 
+=item UTF8 => BOOL
+
+If defined, overrides locale detection to enable or disable UTF-8 mode. If not
+defined then this will be detected from the locale by using Perl's
+C<${^UTF8LOCALE}> variable.
+
 =item writer => OBJECT
 
 An object delegated to for sending strings of terminal control bytes to the
@@ -133,6 +139,8 @@ sub new
    $self->set_input_handle ( $params{input_handle}  ) if $params{input_handle};
    $self->set_output_handle( $params{output_handle} ) if $params{output_handle};
 
+   $self->set_utf8( $params{UTF8} ) if defined $params{UTF8};
+
    $self->set_on_resize( $params{on_resize} ) if $params{on_resize};
    $self->set_on_key   ( $params{on_key}    ) if $params{on_key};
    $self->set_on_mouse ( $params{on_mouse}  ) if $params{on_mouse};
